@@ -28,6 +28,8 @@ OTHER_PLT_SECTIONS="
 .plt.got      ${RELOCATING-0} : { *(.plt.got) }
 .plt.sec      ${RELOCATING-0} : { *(.plt.sec) }
 "
+OTHER_GOT_RELOC_SECTIONS="
+  .rel.tls	${RELOCATING-0} : { *(.rel.tls) }"
 
 # Linux modify the default library search path to first include
 # a 32-bit specific directory.
